from .global_variables import get_n_jobs, get_sound_on
from .logging import setup_logger

__all__ = {
    "logger setup function": "setup_logger",
    "get n_jobs parameter": "get_n_jobs",
    "get if sound on": "get_sound_on",
}
